//-----------------------------------------------------------------------
//  Microsoft Robotics Developer Studio CodePlex Samples
//
//  Copyright (C) 2010 Microsoft Corporation.  All rights reserved.
//
//  This code released under the terms of the Microsoft Public License
//  (MS-PL, http://opensource.org/licenses/ms-pl.html)
//
//  $File: SimulationTutorial1.cs $ $Revision: 1 $
//-----------------------------------------------------------------------

#region Inclusions
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;

using System;
using System.Collections.Generic;

#region Simulation namespaces
using Microsoft.Robotics.Simulation;
using Microsoft.Robotics.Simulation.Engine;
using engineproxy = Microsoft.Robotics.Simulation.Engine.Proxy;
using Microsoft.Robotics.Simulation.Physics;
using Microsoft.Robotics.PhysicalModel;
using System.ComponentModel;
#endregion
#endregion

namespace Robotics.SimulationTutorial1
{
    #region CODECLIP 02-1
    [DisplayName("Simulation Tutorial 1")]
    [Description("Simulation Tutorial 1 Service")]
    [Contract(Contract.Identifier)]
    public class SimulationTutorial1 : DsspServiceBase
    {
        [Partner("Engine",
            Contract = engineproxy.Contract.Identifier,
            CreationPolicy = PartnerCreationPolicy.UseExistingOrCreate)]
        private engineproxy.SimulationEnginePort _engineStub =
            new engineproxy.SimulationEnginePort();

        // Main service port
        [ServicePort("/SimulationTutorial1", AllowMultipleInstances=false)]
        private SimulationTutorial1Operations _mainPort =
            new SimulationTutorial1Operations();

        public SimulationTutorial1(DsspServiceCreationPort creationPort) :
                base(creationPort)
        {
        }
    #endregion

        #region Dmarrage
        protected override void Start()
        {
            base.Start();
            // Orient sim camera view point
            SetupCamera();
            // Add objects (entities) in our simulated world
            PopulateWorld();
        }
        #endregion

        #region Cration de la camera et du monde
        private void SetupCamera()
        {
            // Set up initial view
            CameraView view = new CameraView();
            view.EyePosition = new Vector3(-1.65f, 1.63f, -0.29f);
            view.LookAtPoint = new Vector3(0.0f, 0.0f, 0.0f);
            SimulationEngine.GlobalInstancePort.Update(view);
        }
        #endregion

        private void PopulateWorld()
        {
            AddSky();
            AddGround();
            AddCollada();
        }

        #region Ajoute un ciel
        void AddSky()
        {
            // Add a sky using a static texture. We will use the sky texture
            // to do per pixel lighting on each simulation visual entity
            SkyDomeEntity sky = new SkyDomeEntity("skydome.dds", "sky_diff.dds");
            SimulationEngine.GlobalInstancePort.Insert(sky);

            // Add a directional light to simulate the sun.
            LightSourceEntity sun = new LightSourceEntity();
            sun.State.Name = "Sun";
            sun.Type = LightSourceEntityType.Directional;
            sun.Color = new Vector4(0.8f, 0.8f, 0.8f, 1);
            sun.Direction = new Vector3(0.5f, -.75f, 0.5f);
            SimulationEngine.GlobalInstancePort.Insert(sun);
        }
        #endregion

        #region Ajoute le sol
        void AddGround()
        {
            // create a large horizontal plane, at zero elevation.
            HeightFieldEntity ground = new HeightFieldEntity(
                "simple ground", // name
                "03RamieSc.dds", // texture image
                new MaterialProperties("ground",
                    0.2f, // restitution
                    0.5f, // dynamic friction
                    0.5f) // static friction
                );
            SimulationEngine.GlobalInstancePort.Insert(ground);
        }
        #endregion

        #region Insre une fichier Collada
        void AddCollada()
        {
                case 1:
                    ColladaToMSRS collada1 = new ColladaToMSRS(@"C:\MRDS2008R3\store\media\Collada\robot\robot3_libre.dae");
                    foreach (VisualEntity entity in collada1.Entities)
                    {
                        SimulationEngine.GlobalInstancePort.Insert(entity);
                        entity.EntityState.Pose.Position.Y += 0.17f;
                    }
                    break;

                case 2:
                    ColladaToMSRS collada2 = new ColladaToMSRS(@"C:\MRDS2008R3\store\media\Collada\robot\robot3_mot_nf.dae");
                    foreach (VisualEntity entity in collada2.Entities)
                    {
                        SimulationEngine.GlobalInstancePort.Insert(entity);
                        entity.EntityState.Pose.Position.Y += 0.17f;
                    }
                    break;

                case 3:
                    ColladaToMSRS collada3 = new ColladaToMSRS(@"C:\MRDS2008R3\store\media\Collada\robot\robot.dae");
                    foreach (VisualEntity entity in collada3.Entities)
                    {
                        SimulationEngine.GlobalInstancePort.Insert(entity);
                        entity.EntityState.Pose.Position.Y += 0.17f;
                    }
                    break;

                default:
                    System.Console.WriteLine("gn ?");
                    break;
            }
        }

        #endregion
    }

    public static class Contract
    {
        public const string Identifier = "http://schemas.microsoft.com/robotics/2006/06/simulationtutorial1.html";
    }

    [ServicePort]
    public class SimulationTutorial1Operations : PortSet<DsspDefaultLookup, DsspDefaultDrop>
    {
    }
}
